"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dbl_linked_list_ds_1 = require("dbl-linked-list-ds");
const NodeCancellable_1 = require("../cancellables/NodeCancellable");
const TimerCancellable_1 = require("../cancellables/TimerCancellable");
class Scheduler {
    constructor(cb) {
        this.cb = cb;
        this.isFlushing = false;
        this.queue = new dbl_linked_list_ds_1.LinkedList();
        this.currentTime = Date.now();
        this.onFlush = () => {
            this.isFlushing = false;
            while (this.queue.length > 0) {
                const job = this.queue.shift();
                if (job !== undefined) {
                    job.execute();
                }
            }
        };
    }
    /**
     * Compared to Promise.resolve() asap is a little better —
     * 1. This mimics the exact same behavior of Promise.resolve()
     * 2. It enables efficient cancellation of jobs.
     */
    asap(job) {
        const id = this.queue.add(job);
        this.flush();
        return new NodeCancellable_1.NodeCancellable(this.queue, id);
    }
    /**
     * Some sugar over the native setTimeout() functionality
     */
    delay(job, duration) {
        const id = setTimeout(() => job.execute(), duration);
        return new TimerCancellable_1.TimerCancellable(id);
    }
    now() {
        return Date.now() - this.currentTime;
    }
    flush() {
        if (!this.isFlushing) {
            this.isFlushing = true;
            this.cb(this.onFlush);
        }
    }
}
exports.Scheduler = Scheduler;
